#include <windows.h>
#include <time.h>
#include <conio.h>
#include "main.h"

HANDLE hInst;

int timerId;
struct tm prevtime;
struct tm curtime;
BOOL b_sndopen = FALSE;

int PASCAL WinMain(HANDLE hInstance, HANDLE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    MSG msg;
    if( hPrevInstance == 0 )
    { /* this is the only instance */
        /* initialize things shared by different instances */
        if( !InitApplication(hInstance) )
        {
            return FALSE;
        }
    }
    else
    {
        return 0;
    }
    /* perform initialization that applies only to a specific instance */
    if( !InitInstance(hInstance, nCmdShow) )
    {
        return FALSE;
    }
    while( GetMessage(
               &msg, /* message structure */
               NULL, /* handle of window receiving the message */
               NULL, /* lowest message to examine */
               NULL  /* highest message to examine */
           )
    )
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
    return msg.wParam; /* returns the value from PostQuitMessage */
}

BOOL InitApplication(HANDLE hInstance)
{
    WNDCLASS wc;
    /* fill in window class structure with parameters that describe the main window */
    wc.style = NULL;                         /* class style */
    wc.lpfnWndProc = MainWndProc;            /* function to retrieve message for windows of this class */
    wc.cbClsExtra = 0;                       /* no per-class extra data */
    wc.cbWndExtra = 0;                       /* no per-window extra data */
    wc.hInstance = hInstance;                /* application that owns the class */
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = "AlarmMenu";        /* name of menu resource in the .RC file */
    wc.lpszClassName = "AlarmWClass";     /* name of the class; also used in call to CreateWindow */
    /* register the window class */
    return RegisterClass(&wc);
}

BOOL InitInstance(HANDLE hInstance, int nCmdShow)
{
    HWND hWnd; /* main window handle */
    hInst = hInstance; /* initialize the global hInst */
    /* create a main window for this instance */
    hWnd = CreateWindow(
        "AlarmWClass",                /* name of the class */
        "Alarm Application",   /* text for window title bar */
        WS_OVERLAPPEDWINDOW,              /* window style */
        CW_USEDEFAULT,                   /* horizontal position */
        CW_USEDEFAULT,                   /* vertical position */
        CW_USEDEFAULT,                   /* width */
        CW_USEDEFAULT,                   /* height */
        NULL,                            /* an overlapped window has no parent */
        NULL,                            /* use the window class menu; no specific */
        hInstance,                       /* the instance that owns the window */
        NULL                             /* pointer not needed */
    );
    if( !hWnd ){ /* failure */
        return FALSE;
    }
    //ShowWindow(hWnd, nCmdShow); /* show the window */
    ShowWindow(hWnd, SW_HIDE);
    UpdateWindow(hWnd);         /* send WM_PAINT message */
    return TRUE;
}

long FAR PASCAL MainWndProc(HWND hWnd, unsigned int message, WORD wParam, LONG lParam)
{
    FARPROC lpProcAbout;   /* pointer to the "AboutDlgProc" function */
    int ret;
    time_t ltime;

    switch( message )
    {
    case WM_CREATE:
        /* Set the timer for five-second intervals */
        timerId = SetTimer(hWnd, NULL, 1000, (FARPROC)NULL); // 19/20 of one second;
        if( timerId == 0 )
        { // timer creation failed
            PostMessage(hWnd, WM_DESTROY, 0, 0);
        }
        time(&ltime);
        prevtime = *(localtime(&ltime));
        break;
    case WM_COMMAND:
        if( wParam == IDM_ABOUT )
        {
            lpProcAbout = MakeProcInstance(AboutDlgProc, hInst);
            DialogBox(
                hInst,      /* current instance */
                "AboutBox", /* resource to use */
                hWnd,       /* parent handle */
                lpProcAbout /* "AboutDlgProc" instance address */
            );
        } else { /* message not processed */
            return DefWindowProc(hWnd, message, wParam, lParam); /* passes the message */
        }
        break;
    case WM_TIMER:
        time(&ltime);
        curtime = *(localtime(&ltime));
        if( ( curtime.tm_min == 59 || curtime.tm_min == 0 || curtime.tm_min == 29 ||
          curtime.tm_min == 30 ) && ( curtime.tm_sec >= 55 || curtime.tm_sec == 0 ) )
        {
            if( prevtime.tm_sec != curtime.tm_sec )
            {
                prevtime = curtime;
                if( ( curtime.tm_min == 59 || curtime.tm_min == 29 ) && curtime.tm_sec >= 55 )
                {
                    Beep(392, 10);
                }
                else if( ( curtime.tm_min == 0 || curtime.tm_min == 30 ) && curtime.tm_sec == 0)
                {
                    Beep(523, 18);
                }
            }
        }
        break;
    case WM_DESTROY:
        if( timerId != 0 ) KillTimer(hWnd, timerId);
        if( b_sndopen ) CloseSound();
        PostQuitMessage(0);
        break;
    default: /* message not processed */
        return DefWindowProc(hWnd, message, wParam, lParam);
    }
    return NULL;
}

BOOL FAR PASCAL AboutDlgProc(HWND hDlg, unsigned int message, WORD wParam, LONG lParam)
{
    switch( message )
    {
    case WM_INITDIALOG:
        return TRUE; /* let Windows OS treat it */
    case WM_COMMAND:
        if( wParam == IDOK || wParam == IDCANCEL )
        {
            EndDialog(hDlg, TRUE);
            return TRUE;
        }
        break;
    }
    /* message not processed */
    return FALSE;
}

/* Sounds the speaker for a time specified in microseconds by duration
 * at a pitch specified in hertz by frequency.
 */
void Beep( int frequency, int duration )

{
#if defined( OS2 )
#define INCL_NOCOMMON       /* Use system call for OS/2 */
#define INCL_NOPM
#define INCL_DOSPROCESS
#include <os2.h>
    DosBeep( frequency, duration );
#else
    int control;            /* Do it ourself for DOS */

    /* If frequency is 0, Beep doesn't try to make a sound. It
     * just sleeps for the duration.
     */
    if( frequency )
    {
        /* 75 is about the shortest reliable duration of a sound. */
        if( duration < 75 )
            duration = 75;

        /* Prepare timer by sending 10111100 to port 43. */

        outp( 0x43, 0xb6 );

        /* Divide input frequency by timer ticks per second and
         * write (byte by byte) to timer.
         */
        frequency = (unsigned)(1193180L / frequency);
        outp( 0x42, (char)frequency );
        outp( 0x42, (char)(frequency >> 8) );

        /* Save speaker control byte. */
        control = inp( 0x61 );

        /* Turn on the speaker (with bits 0 and 1). */
        outp( 0x61, control | 0x3 );
    }

    Sleep( (clock_t)duration );

    /* Turn speaker back on if necessary. */

    if( frequency )
        outp( 0x61, control );

#endif              /* DOS version */
}

/* Pauses for a specified number of microseconds. */
void Sleep( clock_t wait )
{
    clock_t goal;

    goal = wait + clock();
    while( goal > clock() )
        ;

}
